package mvd.jester.model;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.mockito.Mockito.mock;
import java.io.IOException;
import java.util.Set;
import java.util.concurrent.ThreadLocalRandom;
import org.junit.Rule;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.rules.TemporaryFolder;


public class TestSystemSetup {
    @Rule
    public TemporaryFolder tf = new TemporaryFolder();

    private static final int NUMBER_OF_RUNS = 1000;

    @Test
    @DisplayName("Check if randomly generated task parameters lie within the specified ranges.")
    public void testRandomTaskSetGeneration() {

        for (int i = 0; i < NUMBER_OF_RUNS; ++i) {
            long numberOfProcessors = ThreadLocalRandom.current().nextLong(2, 8);
            SystemSetup.Builder systemSetupBuilder = new SystemSetup.Builder()//
                    .setNumberOfSegments(1, 7)//
                    .setNumberOfJobs(2, 10)//
                    .setPeriods(100, 1000, 1000)//
                    .setNumberOfProcessors(numberOfProcessors);

            SystemSetup systemSetup = systemSetupBuilder.build();

            assertTrue(systemSetup.getNumberOfProcessors() == numberOfProcessors);

            for (Task t : systemSetup.getTasks()) {
                assertTrue(t.getPeriod() >= 100);
                assertTrue(t.getPeriod() <= 1000);
                assertTrue(t.getDeadline() == t.getPeriod());

                long maxJobWcet = t.getPeriod() / t.getSegments().size();

                for (Segment s : t.getSegments()) {
                    assertTrue(s.getJobWcet() * s.getNumberOfJobs() == s.getTaskletWcet()
                            * s.getNumberOfTasklets());
                    assertTrue(s.getJobWcet() >= 1);
                    assertTrue(s.getJobWcet() <= maxJobWcet);
                    assertTrue(s.getNumberOfJobs() >= 1);
                    assertTrue(s.getNumberOfJobs() <= 10);
                }

                assertTrue(t.getSegments().size() >= 1);
                assertTrue(t.getSegments().size() <= 7);

            }

            assertTrue(systemSetupBuilder.addTask(systemSetup));

            Set<Task> tasks = systemSetup.getTasks();
            systemSetupBuilder.rebuild(systemSetup);
            assertFalse(tasks == systemSetup.getTasks());
        }
    }

    @Test
    @DisplayName("Check Getters and Setters.")
    void testGettersAndSetters() {
        @SuppressWarnings("unchecked")
        Set<Task> t1 = mock(Set.class);

        @SuppressWarnings("unchecked")
        Set<Task> t2 = mock(Set.class);

        SystemSetup systemSetup = new SystemSetup(t1, 2);

        systemSetup.setTasks(t2);
        assertTrue(systemSetup.getTasks() == t2);
    }

    // @Test
    // @DisplayName("Check if parser works correclty.")
    // void testParser() throws IOException {
    // SystemSetup systemSetup = new SystemSetup.Builder().setNumberOfSegments(1, 7)
    // .setNumberOfJobs(2, 10).setPeriods(100, 1000, 1000).build();

    // systemSetup.writeToFile(null);
    // systemSetup.writeToFile(tf.getRoot().getAbsolutePath());
    // }
}
