
package mvd.jester.tests;

import static org.junit.jupiter.api.Assertions.assertTrue;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import mvd.jester.info.SchedulingInfo;
import mvd.jester.info.TerminationInfo.Level;
import mvd.jester.model.SystemSetup;
import mvd.jester.priority.RateMonotonic;

/**
 * TestSchmidMottok
 */
public class TestSchmidMottok {

    @Test
    @DisplayName("Check if the schedulability check returns the correct values.")
    public void testRunSchedulabilityCheck() {
        {
            SystemSetup systemSetup =
                    SystemSetup.readFromFile("src/test/resources/Taskset1.txt", 4);
            SchmidMottok sm = new SchmidMottok(systemSetup);

            SchedulingInfo schedulingInfo = sm.runSchedulabilityCheck(new RateMonotonic());

            assertTrue(schedulingInfo.getTerminationInfos().size() == 4);
            assertTrue(schedulingInfo.checkLevelFail(Level.HIGH) == true);
            assertTrue(schedulingInfo.checkTasksetFeasible() == false);
            assertTrue(schedulingInfo.getFailedTerminationInfo().isPresent());
        }
        {
            SystemSetup systemSetup =
                    SystemSetup.readFromFile("src/test/resources/Taskset1.txt", 8);
            SchmidMottok sm = new SchmidMottok(systemSetup);

            SchedulingInfo schedulingInfo = sm.runSchedulabilityCheck(new RateMonotonic());

            assertTrue(schedulingInfo.getTerminationInfos().size() == 4);
            assertTrue(schedulingInfo.checkLevelFail(Level.HIGH) == true);
            assertTrue(schedulingInfo.checkTasksetFeasible() == false);
            assertTrue(schedulingInfo.getFailedTerminationInfo().isPresent());
        }
        {
            SystemSetup systemSetup =
                    SystemSetup.readFromFile("src/test/resources/Taskset1.txt", 16);
            SchmidMottok sm = new SchmidMottok(systemSetup);

            SchedulingInfo schedulingInfo = sm.runSchedulabilityCheck(new RateMonotonic());

            assertTrue(schedulingInfo.getTerminationInfos().size() == 4);
            assertTrue(schedulingInfo.checkLevelFail(Level.HIGH) == false);
            assertTrue(schedulingInfo.checkTasksetFeasible() == true);
            assertTrue(!schedulingInfo.getFailedTerminationInfo().isPresent());
        }
    }
}
